@@ -29,7 +29,7 @@ from pysqueezecenter.server import Server
 from pysqueezecenter.player import Player
 
 app_path = os.environ['HOME'] + '/.squeezecontrol/'
-app_version = '0.1.2'
+app_version = '0.1.3'
 
 def format_seconds(seconds):
 	seconds = int(seconds)
@@ -49,11 +49,13 @@ def format_seconds(seconds):
 
 class PlayerWindow(QMainWindow):
 	timer = None
+	updating = False
 	timer_count = 5
 	elapsed = 0
-	remaining = 0
+	remaining = -1
 	duration = 0
 	track_info = ''
+	playing = False
 	
 	def __init__(self, parent, player):
 		QMainWindow.__init__(self, parent)
@@ -75,68 +77,93 @@ class PlayerWindow(QMainWindow):
 	def can_skip_backwards(self):
 		return self.player.get_track_path().startswith('file:')
 	
+	def update_info(self):
+		self.playing = self.player.get_mode() == 'play'
+		old_track_info = self.track_info
+		track_info = '<font size="5">%s</font>' % self.player.get_track_title()
+		album = self.player.get_track_album()
+		artist = self.player.get_track_artist()
+		if artist != '':
+			track_info += '<br />%s' % artist
+		if album != '':
+			track_info += '<br />%s' % album
+		self.track_info = track_info.decode('utf-8')
+		if old_track_info != self.track_info:
+			visible = self.can_skip_forwards()
+			self.progress_bar.setVisible(visible)
+			self.remaining_label.setVisible(visible)
+		self.elapsed = self.player.get_time_elapsed()
+		self.duration = self.player.get_track_duration()
+		self.remaining = self.duration - self.elapsed
+		self.timer_count = 1
+		self.updating = False
+	
 	def update_display(self, force_update = False):
-		if force_update or self.timer_count == 5 or self.remaining == 0:
-			old_track_info = self.track_info
-			self.track_info = self.player.get_track_title()
-			album = self.player.get_track_album()
-			if album != '':
-				self.track_info += ' from %s' % album
-			artist = self.player.get_track_artist()
-			if artist != '':
-				self.track_info += ' by %s' % artist
-			self.track_info = self.track_info.decode('utf-8')
-			if old_track_info != self.track_info:
-				visible = self.can_skip_forwards()
-				self.progress_bar.setVisible(visible)
-				self.remaining_label.setVisible(visible)
-			self.elapsed = self.player.get_time_elapsed()
-			if self.remaining_label:
-				self.remaining = self.player.get_time_remaining()
-			if self.progress_bar:
-				self.duration = self.player.get_track_duration()
-			self.timer_count = 0
-		else:
-			self.timer_count += 1
+		if (not self.updating) and (force_update or self.timer_count == 5 or -1 < self.remaining < 1 ):
+			self.updating = True
+			QTimer.singleShot(1, self.update_info)
+		self.timer_count += 1
+		if self.playing:
 			self.elapsed += 1
 			self.remaining -= 1
 		self.track_info_label.setText(self.track_info)
 		self.elapsed_label.setText(format_seconds(self.elapsed))
 		self.remaining_label.setText(format_seconds(self.remaining))
 		if self.can_skip_forwards():
-			position = int(self.elapsed / self.duration * 1000)
+			position = int(self.elapsed / self.duration * 1000) if self.duration else 0
 			self.progress_bar.setValue(position)
 		self.update()
 	
 	def render(self):
-		self.main_widget = QWidget(self)
-		vbox = QVBoxLayout(self.main_widget)
+		# Get power state
+		state = self.player.get_power_state()
+
+		# Add menu
+		self.menu = QMenuBar()
+		self.onoff = self.menu.addAction('Turn on/off')
+		self.onoff.setVisible(True)
+		self.onoff.triggered.connect(self.toggle_power)
+		self.onoff.setText('Turn off' if state else 'Turn on')
+		self.setMenuBar(self.menu)
+		
+		# Set initial widget based on current power state
+		if state:
+			# Update display and make a timer update it every now and then
+			self.render_on_widget()
+			self.update_display()
+			self.start_timer()
+		else:
+			self.render_off_widget()
+
+	def render_on_widget(self):
+		self.on_widget = QWidget()
+		on_vbox = QVBoxLayout(self.on_widget)
 		
 		# Title and artist
 		self.track_info_label = QLabel('')
 		self.track_info_label.setWordWrap(True)
 		self.track_info_label.setAlignment(Qt.AlignCenter)
-		vbox.addWidget(self.track_info_label)
+		on_vbox.addWidget(self.track_info_label)
 		
 		# Time and progress
-		time = QWidget(self.main_widget)
+		time = QWidget()
 		hbox = QHBoxLayout(time)
 		
 		self.elapsed_label = QLabel('')
-		hbox.addWidget(self.elapsed_label)
+		hbox.addWidget(self.elapsed_label, alignment=Qt.AlignCenter)
 		self.progress_bar = QProgressBar()
 		self.progress_bar.setTextVisible(False)
 		self.progress_bar.setRange(0, 1000)
 		hbox.addWidget(self.progress_bar)	
 		self.remaining_label = QLabel('')
-		hbox.addWidget(self.remaining_label)
+		hbox.addWidget(self.remaining_label, alignment=Qt.AlignCenter)
 		if not self.can_skip_forwards():
 			self.progress_bar.setVisible(False)
 			self.remaining_label.setVisible(False)
-		vbox.addWidget(time)
+		on_vbox.addWidget(time)
 		
 		# Main controls
-		controls = QWidget(self.main_widget)
+		controls = QWidget()
 		hbox = QHBoxLayout(controls)
 		
 		prev = QPushButton('<<')
@@ -150,19 +177,39 @@ class PlayerWindow(QMainWindow):
 		next = QPushButton('>>')
 		self.connect(next, SIGNAL('clicked()'), self.next)
 		hbox.addWidget(next)
-		vbox.addWidget(controls)
+		on_vbox.addWidget(controls)
 		
 		# Volume slider
 		vol = QSlider(Qt.Horizontal)
 		vol.setValue(self.player.get_volume())
-		vbox.addWidget(vol)
+		on_vbox.addWidget(vol)
 		vol.valueChanged.connect(self.change_vol)
+
+		self.setCentralWidget(self.on_widget)
+	
+	def render_off_widget(self):
+		self.off_widget = QWidget()
+		off_vbox = QVBoxLayout(self.off_widget)
+		label = QLabel('%s is turned off.' % self.player.get_name())
+		label.setAlignment(Qt.AlignCenter)
+		off_vbox.addWidget(label)
+		button = QPushButton('Turn on')
+		button.clicked.connect(self.toggle_power)
+		off_vbox.addWidget(button)
 		
-		self.setCentralWidget(self.main_widget)
+		self.setCentralWidget(self.off_widget)
 		
-		# Update display and make a timer update it every now and then
-		self.update_display()
-		self.start_timer()
+	def toggle_power(self):
+		state = self.player.get_power_state()
+		self.player.set_power_state(not state)
+		self.onoff.setText('Turn on' if state else 'Turn off')
+		if state:
+			self.clear_timer()
+			self.render_off_widget()
+		else:
+			self.render_on_widget()
+			self.update_display()
+			self.start_timer()
 	
 	def start_timer(self):
 		if self.timer == None:
@@ -175,9 +222,6 @@ class PlayerWindow(QMainWindow):
 			self.timer.stop()
 			self.timer = None
 	
-	def test(self):
-		print 'close'
-	
 	def prev(self):
 		self.player.prev()
 		self.update_display(True)
@@ -190,6 +234,7 @@ class PlayerWindow(QMainWindow):
 		self.player.toggle()
 		mode = self.player.get_mode()
 		self.play_button.setText('Playing' if mode == 'play' else 'Paused')
+		# Is this okay? Changes made by other control devices (remote controls, web interface, etc.) will not show up until the play button is pushed again.
 		if mode == 'play':
 			self.start_timer()
 		else:
@@ -241,6 +286,7 @@ class StartWindow(QMainWindow):
 			button = QPushButton(text)
 			self.connect(button, SIGNAL('clicked()'), action)
 			vbox.addWidget(button)
+		
 		self.setCentralWidget(container)
 	
 	def open_player_selection(self):
@@ -265,7 +311,7 @@ class AboutWindow(QMainWindow):
 	def render(self):
 		container = QWidget()
 		vbox = QVBoxLayout(container)
-		label = QLabel('This version of SqueezeControl was made while listening to cheesy piano music. A huge shoutout to JingleManSweep who made the pysqueezecenter wrapper library for Python. Also thanks to Dag Liodden -- developer of SqueezeControl for Android -- for letting me use the name "SqueezeControl".\n\nCopyright 2010, 2011 JingleManSweep and Mikkel Munch Mortensen. Released under the terms of the GNU General Public License version 3.')
+		label = QLabel('This version of SqueezeControl was made while listening to Mission Control on SomaFM. A huge shoutout to JingleManSweep who made the pysqueezecenter wrapper library for Python. Also thanks to Dag Liodden -- developer of SqueezeControl for Android -- for letting me use the name "SqueezeControl".\n\nCopyright 2010, 2011 JingleManSweep and Mikkel Munch Mortensen. Released under the terms of the GNU General Public License version 3.')
 		label.setWordWrap(True)
 		vbox.addWidget(label)
 		
